"""
This module contains PropertyObject implementation.

There is no need to import this module directly. Use the methods of
the L{Ruleset object <ruleset.Ruleset>} to create and manipulate
PropertyObjects.

    >>> o = r.object_add("/PropertyObjects/Test")
    >>> o.add("Filename=/index.html")
    >>> print o.items
    ['Filename=/index.html']

@var __revision__: Revision identifier.
"""

__revision__ = '%K%'

import plobject
from thisversion import PLDB
import plutil
from plutil import verify_ip

class PropertyObject(plobject.PLObject):
	"""
	PropertyObject Implementation

	Object path: /PropertyObjects

	@cvar Item: PropertyObject Item implementation.
	"""

	__slots__ = ('_prop', '_pldb', '_callback', '_item_added')
	
	base_path = plutil.roConst("/PropertyObjects", "Only the type portion of the path")
	type = plutil.roConst(PLDB.OBJECT_TYPE_PROPERTYOBJECT, "Number representing the type of object")

	def _create_item(self, value):
		if "=" not in value:
			raise ValueError("Format error on string should be prop=value")

		property, val = value.split("=", 1)
		
		item = self.Item(self, property=property, value=val)

		if item._prop.property not in self._pldb.services_prop_list():
			raise ValueError("Unknown property '%s'. Check ruleset.services_prop_list()." % item._prop.property)
		
		return item

	def add(self, value):
		"""
		Add a ProptertyObject Item to this PropertyObject

		@type value: C{str}
		@param value: A string that describes the value of the PropertyObject item
			this should be formatted as I{property=value}

			Wildcards are allowed in value:

				- * matches anything
				- ? matches one arbitrary character
				- % matches any number of non-space characters

			Wildcard-characters can be escaped using \ to match the literal character.

		"""
		plobject.PLObject.add(self, value)

	class Item(plobject.PLObject.Item):
		"""
		PropertyObject Item implementation.
		Should not be instantiated, use L{PropertyObject.add} instead.

		@type property: str
		@type value: str
		"""
		__slots__ = ('_prop', '_parent')
		_Prop = plutil.Prop(property=str, value=str, *plobject._item_props)

		def __str__(self):
			return "%s=%s" % (self._prop.property, self._prop.value)

		value = property(lambda self: self._prop.value, doc="What to match in the property")

		# Make sure to put this one last, since now, unfortunately, property is a reserved name that gets overwritten... :/
		
		property = property(lambda self: self._prop.property, doc="The property matching")
